(*
 * This file is part of Bolt.
 * Copyright (C) 2009 Xavier Clerc.
 *
 * Bolt is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bolt is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *)


(* Definitions *)

class type impl =
  object
    method write : string -> unit
    method close : unit
  end

type t = string -> float option -> Layout.t lazy_t -> impl

let outputs = Hashtbl.create 17

let register name output =
  Hashtbl.replace outputs name output

let get name =
  Hashtbl.find outputs name


(* Predefined outputs *)

let void _ _ _ =
  object
    method write _ = ()
    method close = ()
  end

let outputname s t =
  let dir = Filename.dirname s in
  let base = Filename.basename s in
  let buff = Buffer.create (20 + (String.length s)) in
  String.iter
    (function
      | '%' ->
          let now = Unix.localtime t in
          let millis = int_of_float (mod_float (t *. 1000.) 1000.) in
          let s = Printf.sprintf "%d-%02d-%02d-%02d-%02d-%02d-%03d" (1900 + now.Unix.tm_year) (succ now.Unix.tm_mon) now.Unix.tm_mday now.Unix.tm_hour now.Unix.tm_min now.Unix.tm_sec millis in
          Buffer.add_string buff s
      | c -> Buffer.add_char buff c)
    base;
  Filename.concat dir (Buffer.contents buff)

let write_strings ch l =
  try
    List.iter (fun s -> output_string ch s; output_char ch '\n') l
  with _ -> ()

let open_channel filename h t =
  let ch, reg = match filename with
  | "<stdout>" -> stdout, false
  | "<stderr>" -> stderr, false
  | _ -> (open_out (outputname filename t)), true in
  write_strings ch h;
  ch, reg

let file filename rot layout =
  let now = Unix.gettimeofday () in
  let header, footer, _ = Lazy.force layout in
  try
    let ch, regular = open_channel filename header now in
    let rotate = if regular then rot else None in
    object (self)
      val mutable channel = ch
      val mutable last_rotate = now
      method write s =
        try
          output_string channel s;
          output_char channel '\n';
          flush channel;
          match rotate with
          | Some x ->
              let now = Unix.gettimeofday () in
              if now -. last_rotate >= x then begin
                self#close;
                channel <- fst (open_channel filename header now);
                last_rotate <- now
              end
          | None -> ()
        with _ -> ()
      method close =
        write_strings ch footer;
        close_out_noerr ch
    end
  with _ -> void filename rot layout

let () =
  List.iter
    (fun (x, y) -> register x y)
    [ "void", void ;
      "file", file ]
